import '../../js/deprecate';
import '../../css/DateTime.css';
import '../../css/Text.css';
import React, { useState, createRef, useEffect, useMemo } from 'react';
import { en, ja, zh, format } from './components/Locale';
import { parseStringToDate, parseStringToTime } from './components/utils';
import Calendar from './components/Calendar';
import TimePicker from './components/TimePicker';
import Message from '../../constant/Message';
import '../../css/font.css';
var DateTime = function (_a) {
    var value = _a.value, _b = _a.isDisabled, isDisabled = _b === void 0 ? false : _b, _c = _a.isVisible, isVisible = _c === void 0 ? true : _c, onChange = _a.onChange, _d = _a.locale, locale = _d === void 0 ? 'ja' : _d, _e = _a.dateFormat, dateFormat = _e === void 0 ? 'MM/dd/YYYY' : _e, _f = _a.type, type = _f === void 0 ? 'datetime' : _f, _g = _a.timeFormat, timeFormat = _g === void 0 ? 'HH:mm' : _g;
    var localeObj = ja;
    if (locale === 'en') {
        localeObj = en;
    }
    else if (locale === 'zh') {
        localeObj = zh;
    }
    var validatedValue = useMemo(function () {
        if (!value)
            return new Date();
        if (!(value instanceof Date) || isNaN(value.getTime())) {
            throw new Error(Message.common.INVALID_ARGUMENT);
        }
        return value;
    }, [value]);
    var _h = useState(validatedValue), defaultValue = _h[0], setDefaultValue = _h[1];
    var _j = useState('none'), pickerDisplay = _j[0], setPickerDisplay = _j[1];
    var _k = useState(false), showPickerError = _k[0], setShowPickerError = _k[1];
    var _l = useState(''), dateError = _l[0], setDateError = _l[1];
    var _m = useState('none'), timePickerDisplay = _m[0], setTimePickerDisplay = _m[1];
    var _o = useState(''), inputValue = _o[0], setInputValue = _o[1];
    var _p = useState(value === null ? '' : format(validatedValue, timeFormat)), timeValue = _p[0], setTimeValue = _p[1];
    var _q = useState(value !== null), hasSelection = _q[0], setHasSelection = _q[1];
    var _r = useState(value !== null), hasSelectionTime = _r[0], setHasSelectionTime = _r[1];
    var _s = useState(new Date(validatedValue)), timeDateValue = _s[0], setTimeDateValue = _s[1];
    var _t = useState(isDisabled), isDisableBtn = _t[0], setDisableBtn = _t[1];
    var _u = useState(type), typeDateTime = _u[0], setTypeDateTime = _u[1];
    var wrapperRef = createRef();
    var calendarRef = createRef();
    var timeRef = createRef();
    useEffect(function () {
        setHasSelectionTime(value !== null);
        setHasSelection(value !== null);
    }, [value]);
    var _changeMinutesBy = function (minutes, timeInput) {
        var newTime = new Date(timeDateValue);
        newTime.setSeconds(0);
        newTime.setMinutes(timeDateValue.getMinutes() + minutes);
        newTime.setMonth(timeDateValue.getMonth());
        newTime.setDate(timeDateValue.getDate());
        setTimeDateValue(newTime);
        onChange && onChange(newTime);
        setTimeout(function () {
            setTimeValue(format(newTime, timeFormat));
            timeInput.setSelectionRange(3, 5);
        }, 1);
    };
    var _changeHoursBy = function (hours, timeInput) {
        var newTime = new Date(timeDateValue);
        newTime.setSeconds(0);
        newTime.setHours(timeDateValue.getHours() + hours);
        newTime.setMonth(timeDateValue.getMonth());
        newTime.setDate(timeDateValue.getDate());
        setTimeDateValue(new Date(newTime));
        onChange && onChange(new Date(newTime));
        setTimeout(function () {
            setTimeValue(format(newTime, timeFormat));
            timeInput.setSelectionRange(0, 2);
        }, 1);
    };
    var timeInputKeydownHandler = function (e) {
        var timeTextInput = e.target;
        switch (e.key) {
            case 'Tab':
                if (timeTextInput.selectionStart !== 3 && timeTextInput.selectionEnd !== 5) {
                    e.preventDefault();
                    setTimePickerDisplay('none');
                    timeTextInput.setSelectionRange(3, 5);
                }
                break;
            case 'ArrowLeft':
            case 'Left':
                e.preventDefault();
                timeTextInput.setSelectionRange(0, 2);
                setTimePickerDisplay('none');
                break;
            case 'ArrowRight':
            case 'Right':
                e.preventDefault();
                timeTextInput.setSelectionRange(3, 5);
                setTimePickerDisplay('none');
                break;
            case 'ArrowUp':
            case 'Up':
                e.preventDefault();
                if (timeTextInput.selectionStart && timeTextInput.selectionEnd &&
                    timeTextInput.selectionStart >= 2 && timeTextInput.selectionStart <= 5) {
                    _changeMinutesBy(1, e.target);
                }
                else {
                    _changeHoursBy(1, e.target);
                }
                setTimePickerDisplay('none');
                break;
            case 'ArrowDown':
            case 'Down':
                e.preventDefault();
                if (timeTextInput.selectionStart && timeTextInput.selectionEnd &&
                    timeTextInput.selectionStart >= 2 && timeTextInput.selectionStart <= 5) {
                    _changeMinutesBy(-1, e.target);
                }
                else {
                    _changeHoursBy(-1, e.target);
                }
                setTimePickerDisplay('none');
                break;
            default:
                if (!/[0-9]/.test(e.key)) {
                    e.preventDefault();
                }
                break;
        }
    };
    useEffect(function () {
        if (defaultValue !== validatedValue) {
            setDefaultValue(validatedValue);
            setInputValue(format(validatedValue, dateFormat));
            setTimeValue(format(validatedValue, timeFormat));
            setTimeDateValue(new Date(validatedValue));
        }
        else {
            var newTimeDateValue = new Date(timeDateValue);
            var setNewTimeDateValue = false;
            if (validatedValue.getDate() !== timeDateValue.getDate()) {
                newTimeDateValue.setDate(validatedValue.getDate());
                setNewTimeDateValue = true;
            }
            if (validatedValue.getMonth() !== timeDateValue.getMonth()) {
                newTimeDateValue.setMonth(validatedValue.getMonth());
                setNewTimeDateValue = true;
            }
            if (validatedValue.getFullYear() !== timeDateValue.getFullYear()) {
                newTimeDateValue.setFullYear(validatedValue.getFullYear());
                setNewTimeDateValue = true;
            }
            if (setNewTimeDateValue && pickerDisplay === 'none') {
                setTimeDateValue(newTimeDateValue);
            }
            if (!hasSelection) {
                setInputValue('');
            }
            else if (inputValue !== dateFormat) {
                var newInputValue = format(validatedValue, dateFormat);
                if (newInputValue === dateFormat) {
                    setInputValue(dateFormat);
                    setDateError(Message.datetime.INVALID_DATE);
                    setShowPickerError(true);
                }
                else if (!showPickerError) {
                    setInputValue(newInputValue);
                }
            }
            if (typeof isDisabled !== 'boolean') {
                setDisableBtn(false);
            }
            else {
                setDisableBtn(isDisabled);
            }
        }
    }, [dateFormat, defaultValue, hasSelection, pickerDisplay, timeDateValue, timeFormat, validatedValue, isDisabled, inputValue, showPickerError]);
    if (typeDateTime !== 'datetime' && typeDateTime !== 'date' && typeDateTime !== 'time') {
        setTypeDateTime('datetime');
    }
    if (isVisible) {
        return (React.createElement("div", { className: "date-time-container", ref: wrapperRef },
            (typeDateTime === 'datetime' || typeDateTime === 'date') &&
                React.createElement("div", { className: "date-container" },
                    React.createElement("div", { className: "text-input-container", key: "".concat(dateError) },
                        React.createElement("input", { type: "text", className: "kuc-input-text text-input", disabled: isDisableBtn, onFocus: function (e) {
                                // if (showPickerError) {
                                //   setHasSelection(false);
                                // }
                                if (showPickerError || !hasSelection) {
                                    setPickerDisplay('block');
                                    return;
                                }
                                var temporary = e.target.value ? new Date(parseStringToDate(e.target.value, dateFormat)) : new Date();
                                var dateValue = e.target.value ? new Date(parseStringToDate(e.target.value, dateFormat)) : new Date();
                                temporary.setSeconds(timeDateValue.getSeconds());
                                temporary.setMinutes(timeDateValue.getMinutes());
                                temporary.setHours(timeDateValue.getHours());
                                temporary.setDate(temporary.getDate() - 1);
                                dateValue.setSeconds(timeDateValue.getSeconds());
                                dateValue.setMinutes(timeDateValue.getMinutes());
                                dateValue.setHours(timeDateValue.getHours());
                                setTimeDateValue(temporary);
                                setTimeout(function () {
                                    setPickerDisplay('block');
                                    setTimePickerDisplay('none');
                                    setTimeDateValue(dateValue);
                                }, 1);
                            }, value: value !== null ? inputValue : '', onBlur: function (e) {
                                var tempDate = parseStringToDate(e.target.value, dateFormat);
                                var returnDate = null;
                                if (!e.target.value) {
                                    setHasSelection(false);
                                }
                                else if (tempDate instanceof Date && !isNaN(tempDate)) {
                                    returnDate = new Date(validatedValue);
                                    returnDate.setFullYear(tempDate.getFullYear(), tempDate.getMonth(), tempDate.getDate());
                                    setShowPickerError(false);
                                }
                                else if (e.target.value) {
                                    setDateError(Message.datetime.INVALID_DATE);
                                    setShowPickerError(true);
                                }
                                var relatedTarget = e.relatedTarget ||
                                    e.explicitOriginalTarget ||
                                    document.activeElement; // IE11
                                var calendar = calendarRef.current;
                                if (relatedTarget !== calendar && !calendar.contains(relatedTarget)) {
                                    setPickerDisplay('none');
                                    if (timeValue === '' || value === null) {
                                        setHasSelectionTime(false);
                                    }
                                    if (returnDate) {
                                        onChange && onChange(returnDate);
                                        setShowPickerError(false);
                                    }
                                }
                            }, onKeyDown: function (e) {
                                if (e.key === 'Tab') {
                                    setPickerDisplay('none');
                                }
                            }, onChange: function (e) {
                                setInputValue(e.target.value);
                            } })),
                    (dateError && showPickerError) &&
                        React.createElement("div", { className: "label-error" },
                            React.createElement("span", null, dateError)),
                    !isDisableBtn &&
                        React.createElement(Calendar, { calRef: calendarRef, pickerDisplay: pickerDisplay, date: timeDateValue, locale: localeObj, hasSelection: hasSelection, onDateClick: function (calendarDate, previousDate) {
                                var tempDate;
                                if (previousDate) {
                                    tempDate = new Date(previousDate);
                                }
                                else {
                                    tempDate = new Date();
                                }
                                if (calendarDate) {
                                    if (validatedValue) {
                                        tempDate = new Date(validatedValue);
                                    }
                                    tempDate.setFullYear(calendarDate.getFullYear(), calendarDate.getMonth(), calendarDate.getDate());
                                    tempDate.setHours(timeDateValue.getHours());
                                    tempDate.setMinutes(timeDateValue.getMinutes());
                                    tempDate.setSeconds(0);
                                    if (timeValue === '' || value === null) {
                                        setHasSelectionTime(false);
                                    }
                                    onChange && onChange(tempDate);
                                    setHasSelection(true);
                                    setShowPickerError(false);
                                }
                                else if (previousDate) {
                                    onChange && onChange(null);
                                    setInputValue('');
                                    setShowPickerError(false);
                                }
                                setPickerDisplay('none');
                            } })),
            (typeDateTime === 'datetime' || typeDateTime === 'time') &&
                React.createElement("div", { className: "time-container" },
                    React.createElement("input", { type: "text", disabled: isDisableBtn, maxLength: 5, key: 1, className: "kuc-input-text text-input time", onClick: function (e) {
                            var timeTextInput = e.target;
                            if (timeTextInput.selectionStart &&
                                (timeTextInput.selectionStart >= 2 && timeTextInput.selectionStart <= 5)) {
                                timeTextInput.setSelectionRange(3, 5);
                            }
                            else {
                                timeTextInput.setSelectionRange(0, 2);
                            }
                            setTimePickerDisplay('flex');
                            setPickerDisplay('none');
                        }, onFocus: function (e) {
                            var timeInput = e.target;
                            setTimeout(function () {
                                timeInput.setSelectionRange(0, 2);
                                e.preventDefault();
                                e.stopPropagation();
                            }, 1);
                            setTimePickerDisplay('flex');
                            setPickerDisplay('none');
                        }, onBlur: function (e) {
                            var relatedTarget = e.relatedTarget ||
                                e.explicitOriginalTarget ||
                                document.activeElement; // IE11
                            var timePicker = timeRef.current;
                            if (relatedTarget !== timePicker && !timePicker.contains(relatedTarget)) {
                                setTimePickerDisplay('none');
                            }
                        }, value: (value !== null) && hasSelectionTime ? timeValue : '', onChange: function (e) {
                            var timeTextInput = e.target;
                            var newTime = parseStringToTime(timeTextInput.value);
                            if (!newTime) {
                                newTime = new Date(timeDateValue);
                            }
                            else {
                                newTime.setDate(timeDateValue.getDate());
                                newTime.setMonth(timeDateValue.getMonth());
                                newTime.setFullYear(timeDateValue.getFullYear());
                            }
                            if (timeTextInput.selectionStart &&
                                timeTextInput.selectionStart >= 3 && timeTextInput.selectionStart <= 5) {
                                // minutes are being edited
                                var previousMinutes = void 0;
                                if (timeDateValue.getMinutes() > 10) {
                                    previousMinutes = ('' + timeDateValue.getMinutes())[1];
                                }
                                else {
                                    previousMinutes = ('' + timeDateValue.getMinutes());
                                }
                                if (parseInt(previousMinutes, 10) > 5) {
                                    previousMinutes = '0';
                                }
                                newTime.setMinutes(parseInt(previousMinutes + '' + newTime.getMinutes(), 10));
                                timeTextInput.value = format(newTime, 'HH:mm');
                                timeTextInput.setSelectionRange(3, 5);
                            }
                            else {
                                // hours are being edited
                                var previousHours = void 0;
                                if (timeDateValue.getHours() > 10) {
                                    previousHours = ('' + timeDateValue.getHours())[1];
                                }
                                else if (timeDateValue.getHours() === 10) {
                                    previousHours = ('' + timeDateValue.getHours())[0];
                                }
                                else {
                                    previousHours = ('' + timeDateValue.getHours());
                                }
                                if (parseInt(previousHours, 10) > 2) {
                                    previousHours = '0';
                                }
                                newTime.setHours(parseInt(previousHours + '' + newTime.getHours(), 10));
                                timeTextInput.value = format(newTime, 'HH:mm');
                                timeTextInput.setSelectionRange(0, 2);
                            }
                            newTime.setSeconds(0);
                            newTime.setMonth(timeDateValue.getMonth());
                            newTime.setDate(timeDateValue.getDate());
                            setTimeValue(format(newTime, timeFormat));
                            setTimeDateValue(new Date(newTime));
                            onChange && onChange(new Date(newTime));
                            setTimePickerDisplay('none');
                        }, onKeyDown: function (e) {
                            timeInputKeydownHandler(e);
                        } }),
                    !isDisableBtn &&
                        React.createElement(TimePicker, { timeRef: timeRef, pickerDisplay: timePickerDisplay, onTimeClick: function (timePickerDate) {
                                setHasSelectionTime(true);
                                var tempDate = new Date();
                                if (timeDateValue)
                                    tempDate = new Date(timeDateValue);
                                value && tempDate.setDate(value.getDate());
                                value && tempDate.setMonth(value.getMonth());
                                tempDate.setHours(timePickerDate.getHours(), timePickerDate.getMinutes());
                                tempDate.setSeconds(0);
                                onChange && onChange(tempDate);
                                setTimePickerDisplay('none');
                            } }))));
    }
    return React.createElement("div", null);
};
export default DateTime;
export { Calendar };
export { en, zh, ja, format, getSeperator, } from './components/Locale';
export { availableLocales } from './components/DateTimeConstant';
